@@ -51,13 +51,12 @@
 #include <utils/rls.h>
 
 #include "compat/compat.h"
+#include "chunk_insert_state.h"
 #include "copy.h"
 #include "cross_module_fn.h"
 #include "dimension.h"
 #include "hypertable.h"
 #include "indexing.h"
-#include "nodes/chunk_dispatch/chunk_dispatch.h"
-#include "nodes/chunk_dispatch/chunk_insert_state.h"
 #include "subspace_store.h"
 
 /*
@@ -140,6 +139,7 @@ typedef struct TSCopyMultiInsertInfo
 	CommandId mycid;		  /* Command Id used for COPY */
 	int ti_options;			  /* table insert options */
 	Hypertable *ht;			  /* The hypertable for the inserts */
+	bool has_continuous_aggregate;
 } TSCopyMultiInsertInfo;
 
 /*
@@ -274,7 +274,17 @@ TSCopyMultiInsertBufferInit(TSCopyMultiInsertInfo *miinfo, ChunkInsertState *cis
 			bool sort = ts_guc_enable_direct_compress_copy_sort_batches &&
 						!ts_guc_enable_direct_compress_copy_client_sorted;
 			buffer->compressor =
-				ts_cm_functions->compressor_init(cis->rel, &buffer->bulk_writer, sort);
+				ts_cm_functions->compressor_init(cis->rel,
+												 &buffer->bulk_writer,
+												 sort,
+												 ts_guc_direct_compress_copy_tuple_sort_limit);
+
+			if (miinfo->has_continuous_aggregate)
+			{
+				ts_cm_functions->compressor_set_invalidation(buffer->compressor,
+															 miinfo->ht,
+															 RelationGetRelid(cis->rel));
+			}
 
 			/*
 			 * The sorting done in the compressor is only a local sort for the
@@ -355,6 +365,7 @@ TSCopyMultiInsertInfoInit(TSCopyMultiInsertInfo *miinfo, ResultRelInfo *rri,
 	miinfo->mycid = mycid;
 	miinfo->ti_options = ti_options;
 	miinfo->ht = ht;
+	miinfo->has_continuous_aggregate = ts_hypertable_has_continuous_aggregates(ht->fd.id);
 }
 
 /*
@@ -482,6 +493,19 @@ TSCopyMultiInsertBufferFlush(TSCopyMultiInsertInfo *miinfo, TSCopyMultiInsertBuf
 								 NULL /* transition capture */);
 		}
 
+		if (miinfo->has_continuous_aggregate && !ts_guc_enable_cagg_wal_based_invalidation)
+		{
+			bool should_free;
+			HeapTuple tuple = ExecFetchSlotHeapTuple(slots[i], false, &should_free);
+			ts_cm_functions->continuous_agg_dml_invalidate(miinfo->ht->fd.id,
+														   resultRelInfo->ri_RelationDesc,
+														   tuple,
+														   NULL,
+														   false);
+			if (should_free)
+				heap_freetuple(tuple);
+		}
+
 		ExecClearTuple(slots[i]);
 	}
 
@@ -582,6 +606,7 @@ TSCopyMultiInsertInfoFlush(TSCopyMultiInsertInfo *miinfo, ChunkInsertState *cur_
 	ListCell *lc;
 
 	current_multi_insert_buffers = hash_get_num_entries(miinfo->multiInsertBuffers);
+	int current_chunk_id = cur_cis ? cur_cis->chunk_id : 0;
 
 	/* Create a list of buffers that can be sorted by usage */
 	hash_seq_init(&status, miinfo->multiInsertBuffers);
@@ -608,7 +633,7 @@ TSCopyMultiInsertInfoFlush(TSCopyMultiInsertInfo *miinfo, ChunkInsertState *cur_
 			 * Reduce active multi-insert buffers. However, the current used buffer
 			 * should not be deleted because it might reused for the next insert.
 			 */
-			if (cur_cis == NULL || flushed_chunk_id != cur_cis->chunk_id)
+			if (current_chunk_id == 0 || flushed_chunk_id != current_chunk_id)
 			{
 				TSCopyMultiInsertBufferCleanup(miinfo, buffer);
 				hash_search(miinfo->multiInsertBuffers, &flushed_chunk_id, HASH_REMOVE, &found);
@@ -740,40 +765,67 @@ copy_table_to_chunk_error_callback(void *arg)
 	errcontext("copying from table %s", RelationGetRelationName(scandesc->rs_rd));
 }
 
-/*
- * Tests if there are other before insert row triggers besides the
- * ts_insert_blocker trigger.
- */
-static inline bool
-has_other_before_insert_row_trigger_than_ts(ResultRelInfo *resultRelInfo)
+static TSCopyInsertMethod
+choose_copy_method(Hypertable *ht, CopyChunkState *ccstate, ResultRelInfo *resultRelInfo)
 {
-	TriggerDesc *trigdesc = resultRelInfo->ri_TrigDesc;
-	int i;
+	/*
+	 * Multi-insert buffers (TS_CIM_MULTI_CONDITIONAL) can only be used if no triggers are
+	 * defined on the target table. Otherwise, the tuples may be inserted in an out-of-order
+	 * manner, which might violate the semantics of the triggers. So, they are inserted
+	 * tuple-per-tuple (TS_CIM_SINGLE). However, the ts_block trigger on the hypertable can
+	 * be ignored.
+	 */
 
-	if (trigdesc == NULL)
-		return false;
+	/* Before INSERT Triggers */
+	bool has_before_insert_row_trig =
+		(resultRelInfo->ri_TrigDesc && resultRelInfo->ri_TrigDesc->trig_insert_before_row);
 
-	if (!trigdesc->trig_insert_before_row)
-		return false;
+	/* Instead of INSERT Triggers */
+	bool has_instead_insert_row_trig =
+		(resultRelInfo->ri_TrigDesc && resultRelInfo->ri_TrigDesc->trig_insert_instead_row);
 
-	for (i = 0; i < trigdesc->numtriggers; i++)
+	bool has_after_insert_statement_trig =
+		(resultRelInfo->ri_TrigDesc && resultRelInfo->ri_TrigDesc->trig_insert_new_table);
+
+	/* Depending on the configured trigger, enable or disable the multi-insert buffers */
+	if (has_after_insert_statement_trig || has_before_insert_row_trig ||
+		has_instead_insert_row_trig)
 	{
-		Trigger *trigger = &trigdesc->triggers[i];
-		if (!TRIGGER_TYPE_MATCHES(trigger->tgtype,
-								  TRIGGER_TYPE_ROW,
-								  TRIGGER_TYPE_BEFORE,
-								  TRIGGER_TYPE_INSERT))
-			continue;
-
-		/* Ignore the ts_insert_block trigger */
-		if (strncmp(trigger->tgname, INSERT_BLOCKER_NAME, NAMEDATALEN) == 0)
-			continue;
-
-		/* At least one trigger exists */
-		return true;
+		ereport(DEBUG1,
+				(errmsg("Using normal unbuffered copy operation (TS_CIM_SINGLE) "
+						"because triggers are defined on the destination table.")));
+		if (ts_guc_enable_direct_compress_copy)
+			ereport(WARNING,
+					(errmsg("disabling direct compress copy due to presence of triggers on the "
+							"destination table")));
+		return TS_CIM_SINGLE;
 	}
 
-	return false;
+	if (TS_HYPERTABLE_HAS_COMPRESSION_ENABLED(ht) && ts_guc_enable_direct_compress_copy)
+	{
+		if (ts_indexing_relation_has_primary_or_unique_index(ccstate->rel))
+		{
+			ereport(WARNING,
+					(errmsg("disabling direct compress because the destination table has unique "
+							"constraints")));
+		}
+		else if (resultRelInfo->ri_TrigDesc && resultRelInfo->ri_TrigDesc->numtriggers > 1)
+		{
+			ereport(WARNING,
+					(errmsg(
+						"disabling direct compress because the destination table has triggers")));
+		}
+		else
+		{
+			ccstate->ctr->create_compressed_chunk = true;
+			ereport(DEBUG1, (errmsg("Using compressed copy operation (TS_CIM_COMPRESSION).")));
+			return TS_CIM_COMPRESSION;
+		}
+	}
+
+	ereport(DEBUG1,
+			(errmsg("Using optimized multi-buffer copy operation (TS_CIM_MULTI_CONDITIONAL).")));
+	return TS_CIM_MULTI_CONDITIONAL;
 }
 
 /*
@@ -799,9 +851,6 @@ copyfrom(CopyChunkState *ccstate, ParseState *pstate, Hypertable *ht, MemoryCont
 	int ti_options = 0;							   /* start with default options for insert */
 	BulkInsertState bistate = NULL;
 	uint64 processed = 0;
-	bool has_before_insert_row_trig;
-	bool has_instead_insert_row_trig;
-	bool has_after_insert_statement_trig;
 	ExprState *qualexpr = NULL;
 
 	Assert(pstate->p_rtable);
@@ -907,11 +956,11 @@ copyfrom(CopyChunkState *ccstate, ParseState *pstate, Hypertable *ht, MemoryCont
 #endif
 	ExecInitResultRelation(estate, resultRelInfo, 1);
 
-	CheckValidResultRelCompat(resultRelInfo, CMD_INSERT, NIL);
+	CheckValidResultRelCompat(resultRelInfo, CMD_INSERT, ONCONFLICT_NONE, NIL);
 
 	ExecOpenIndices(resultRelInfo, false);
 
-	ccstate->ctr = ts_chunk_tuple_routing_create(estate, resultRelInfo);
+	ccstate->ctr = ts_chunk_tuple_routing_create(estate, ht, resultRelInfo);
 
 	singleslot = table_slot_create(resultRelInfo->ri_RelationDesc, &estate->es_tupleTable);
 
@@ -956,48 +1005,8 @@ copyfrom(CopyChunkState *ccstate, ParseState *pstate, Hypertable *ht, MemoryCont
 		error_context_stack = &errcallback;
 	}
 
-	/*
-	 * Multi-insert buffers (TS_CIM_MULTI_CONDITIONAL) can only be used if no triggers are
-	 * defined on the target table. Otherwise, the tuples may be inserted in an out-of-order
-	 * manner, which might violate the semantics of the triggers. So, they are inserted
-	 * tuple-per-tuple (TS_CIM_SINGLE). However, the ts_block trigger on the hypertable can
-	 * be ignored.
-	 */
+	insertMethod = choose_copy_method(ht, ccstate, resultRelInfo);
 
-	/* Before INSERT Triggers */
-	has_before_insert_row_trig = has_other_before_insert_row_trigger_than_ts(resultRelInfo);
-
-	/* Instead of INSERT Triggers */
-	has_instead_insert_row_trig =
-		(resultRelInfo->ri_TrigDesc && resultRelInfo->ri_TrigDesc->trig_insert_instead_row);
-
-	has_after_insert_statement_trig =
-		(resultRelInfo->ri_TrigDesc && resultRelInfo->ri_TrigDesc->trig_insert_new_table);
-
-	/* Depending on the configured trigger, enable or disable the multi-insert buffers */
-	if (has_after_insert_statement_trig || has_before_insert_row_trig ||
-		has_instead_insert_row_trig)
-	{
-		insertMethod = TS_CIM_SINGLE;
-		ereport(DEBUG1,
-				(errmsg("Using normal unbuffered copy operation (TS_CIM_SINGLE) "
-						"because triggers are defined on the destination table.")));
-	}
-	else if (TS_HYPERTABLE_HAS_COMPRESSION_ENABLED(ht) &&
-			 !ts_indexing_relation_has_primary_or_unique_index(ccstate->rel) &&
-			 ts_guc_enable_direct_compress_copy)
-	{
-		insertMethod = TS_CIM_COMPRESSION;
-		ccstate->ctr->create_compressed_chunk = true;
-		ereport(DEBUG1, (errmsg("Using compressed copy operation (TS_CIM_COMPRESSION).")));
-	}
-	else
-	{
-		insertMethod = TS_CIM_MULTI_CONDITIONAL;
-		ereport(DEBUG1,
-				(errmsg(
-					"Using optimized multi-buffer copy operation (TS_CIM_MULTI_CONDITIONAL).")));
-	}
 	TSCopyMultiInsertInfoInit(&multiInsertInfo,
 							  resultRelInfo,
 							  ccstate,
