@@ -10,9 +10,8 @@
 
 typedef enum CustomType
 {
-	CUSTOM_TYPE_TS_INTERVAL = 0,
-	CUSTOM_TYPE_COMPRESSED_DATA,
-	CUSTOM_TYPE_SEGMENT_META_MIN_MAX,
+	CUSTOM_TYPE_COMPRESSED_DATA = 0,
+	CUSTOM_TYPE_BLOOM1,
 
 	_CUSTOM_TYPE_MAX_INDEX
 } CustomType;
