@@ -6,7 +6,7 @@
 #include <postgres.h>
 #include <access/heapam.h>
 #include <access/htup.h>
-#include <catalog/namespace.h>
+#include <catalog/pg_namespace_d.h>
 #include <catalog/pg_proc.h>
 #include <catalog/pg_type.h>
 #include <miscadmin.h>
@@ -258,6 +258,38 @@ static FuncInfo funcinfo[] = {
 		.group_estimate = time_bucket_group_estimate,
 		.sort_transform = time_bucket_sort_transform,
 	},
+	{
+		.origin = ORIGIN_TIMESCALE,
+		.is_bucketing_func = true,
+		.allowed_in_cagg_definition = true,
+		.funcname = "time_bucket",
+		.nargs = 2,
+		.arg_types = { INTERVALOID, UUIDOID },
+		.group_estimate = time_bucket_group_estimate,
+		.sort_transform = time_bucket_sort_transform,
+	},
+	/* Interval Bucket with origin */
+	{
+		.origin = ORIGIN_TIMESCALE,
+		.is_bucketing_func = true,
+		.allowed_in_cagg_definition = true,
+		.funcname = "time_bucket",
+		.nargs = 3,
+		.arg_types = { INTERVALOID, UUIDOID, TIMESTAMPTZOID },
+		.group_estimate = time_bucket_group_estimate,
+		.sort_transform = time_bucket_sort_transform,
+	},
+	/* Interval Bucket with offset */
+	{
+		.origin = ORIGIN_TIMESCALE,
+		.is_bucketing_func = true,
+		.allowed_in_cagg_definition = true,
+		.funcname = "time_bucket",
+		.nargs = 3,
+		.arg_types = { INTERVALOID, UUIDOID, INTERVALOID },
+		.group_estimate = time_bucket_group_estimate,
+		.sort_transform = time_bucket_sort_transform,
+	},
 	{
 		.origin = ORIGIN_TIMESCALE,
 		.is_bucketing_func = true,
@@ -537,7 +569,6 @@ initialize_func_info()
 	};
 	Oid extension_nsp = ts_extension_schema_oid();
 	Oid experimental_nsp = get_namespace_oid(ts_experimental_schema_name(), false);
-	Oid pg_nsp = get_namespace_oid("pg_catalog", false);
 	HeapTuple tuple;
 	Relation rel;
 
@@ -551,7 +582,7 @@ initialize_func_info()
 	for (size_t i = 0; i < _MAX_CACHE_FUNCTIONS; i++)
 	{
 		FuncInfo *finfo = &funcinfo[i];
-		Oid namespaceoid = pg_nsp;
+		Oid namespaceoid = PG_CATALOG_NAMESPACE;
 		oidvector *paramtypes = buildoidvector(finfo->arg_types, finfo->nargs);
 		FuncEntry *fentry;
 		bool hash_found;
