@@ -71,6 +71,11 @@ static const struct config_enum_entry compress_truncate_behaviour_options[] = {
 bool ts_guc_enable_direct_compress_copy = false;
 bool ts_guc_enable_direct_compress_copy_sort_batches = true;
 bool ts_guc_enable_direct_compress_copy_client_sorted = false;
+int ts_guc_direct_compress_copy_tuple_sort_limit = 100000;
+bool ts_guc_enable_direct_compress_insert = false;
+bool ts_guc_enable_direct_compress_insert_sort_batches = true;
+bool ts_guc_enable_direct_compress_insert_client_sorted = false;
+int ts_guc_direct_compress_insert_tuple_sort_limit = 10000;
 bool ts_guc_enable_deprecation_warnings = true;
 bool ts_guc_enable_optimizations = true;
 bool ts_guc_restoring = false;
@@ -90,6 +95,7 @@ TSDLLEXPORT bool ts_guc_enable_cagg_sort_pushdown = true;
 #endif
 TSDLLEXPORT bool ts_guc_enable_cagg_watermark_constify = true;
 TSDLLEXPORT int ts_guc_cagg_max_individual_materializations = 10;
+TSDLLEXPORT bool ts_guc_enable_cagg_wal_based_invalidation = false;
 TSDLLEXPORT int ts_guc_cagg_wal_batch_size = 10000;
 TSDLLEXPORT int ts_guc_cagg_low_work_mem = GUC_CAGG_LOW_WORK_MEM_VALUE;
 TSDLLEXPORT int ts_guc_cagg_high_work_mem = GUC_CAGG_HIGH_WORK_MEM_VALUE;
@@ -108,11 +114,15 @@ TSDLLEXPORT bool ts_guc_enable_compression_indexscan = false;
 TSDLLEXPORT bool ts_guc_enable_bulk_decompression = true;
 TSDLLEXPORT bool ts_guc_auto_sparse_indexes = true;
 TSDLLEXPORT bool ts_guc_enable_sparse_index_bloom = true;
+
+TSDLLEXPORT bool ts_guc_read_legacy_bloom1_v1 = false;
+
 bool ts_guc_enable_chunk_skipping = false;
 TSDLLEXPORT bool ts_guc_enable_segmentwise_recompression = true;
+TSDLLEXPORT bool ts_guc_enable_in_memory_recompression = true;
 TSDLLEXPORT bool ts_guc_enable_exclusive_locking_recompression = false;
 TSDLLEXPORT bool ts_guc_enable_bool_compression = true;
-TSDLLEXPORT bool ts_guc_enable_uuid_compression = false;
+TSDLLEXPORT bool ts_guc_enable_uuid_compression = true;
 TSDLLEXPORT int ts_guc_compression_batch_size_limit = 1000;
 TSDLLEXPORT bool ts_guc_compression_enable_compressor_batch_limit = false;
 TSDLLEXPORT CompressTruncateBehaviour ts_guc_compress_truncate_behaviour = COMPRESS_TRUNCATE_ONLY;
@@ -141,6 +151,8 @@ bool ts_guc_enable_tss_callbacks = true;
 TSDLLEXPORT bool ts_guc_enable_delete_after_compression = false;
 TSDLLEXPORT bool ts_guc_enable_merge_on_cagg_refresh = false;
 
+bool ts_guc_enable_partitioned_hypertables = false;
+
 /* default value of ts_guc_max_open_chunks_per_insert and
  * ts_guc_max_cached_chunks_per_hypertable will be set as their respective boot-value when the
  * GUC mechanism starts up */
@@ -471,6 +483,70 @@ _guc_init(void)
 							 NULL,
 							 NULL);
 
+	DefineCustomIntVariable(MAKE_EXTOPTION("direct_compress_copy_tuple_sort_limit"),
+							"Number of tuples that can be sorted at once in a COPY operation",
+							"This is mainly used to keep the memory footprint down for "
+							"operations like importing large amounts of data in "
+							"single transaction. Setting this to 0 would make it unlimited.",
+							&ts_guc_direct_compress_copy_tuple_sort_limit,
+							100000,
+							0,
+							2147483647,
+							PGC_USERSET,
+							0,
+							NULL,
+							NULL,
+							NULL);
+
+	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_direct_compress_insert"),
+							 "Enable direct compression during INSERT",
+							 "Enable experimental support for direct compression during INSERT",
+							 &ts_guc_enable_direct_compress_insert,
+							 false,
+							 PGC_USERSET,
+							 0,
+							 NULL,
+							 NULL,
+							 NULL);
+
+	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_direct_compress_insert_sort_batches"),
+							 "Enable batch sorting during direct compress INSERT",
+							 NULL,
+							 &ts_guc_enable_direct_compress_insert_sort_batches,
+							 true,
+							 PGC_USERSET,
+							 0,
+							 NULL,
+							 NULL,
+							 NULL);
+
+	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_direct_compress_insert_client_sorted"),
+							 "Enable direct compress INSERT with presorted data",
+							 "Correct handling of data sorting by the user is required for this "
+							 "option.",
+							 &ts_guc_enable_direct_compress_insert_client_sorted,
+							 false,
+							 PGC_USERSET,
+							 0,
+							 NULL,
+							 NULL,
+							 NULL);
+
+	DefineCustomIntVariable(MAKE_EXTOPTION("direct_compress_insert_tuple_sort_limit"),
+							"Number of tuples that can be sorted at once in an INSERT operation",
+							"This is mainly used to keep the memory footprint down for "
+							"operations like importing large amounts of data in "
+							"single transaction. Setting this to 0 would make it unlimited.",
+							&ts_guc_direct_compress_insert_tuple_sort_limit,
+							10000,
+							0,
+							2147483647,
+							PGC_USERSET,
+							0,
+							NULL,
+							NULL,
+							NULL);
+
 	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_optimizations"),
 							 "Enable TimescaleDB query optimizations",
 							 NULL,
@@ -831,6 +907,16 @@ _guc_init(void)
 							 NULL,
 							 NULL,
 							 NULL);
+	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_in_memory_recompression"),
+							 "Enable in-memory recompression functionality",
+							 "Enable in-memory recompression",
+							 &ts_guc_enable_in_memory_recompression,
+							 true,
+							 PGC_USERSET,
+							 0,
+							 NULL,
+							 NULL,
+							 NULL);
 	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_exclusive_locking_recompression"),
 							 "Enable exclusive locking recompression",
 							 "Enable getting exclusive lock on chunk during segmentwise "
@@ -858,7 +944,7 @@ _guc_init(void)
 							 "Enable uuid compression functionality",
 							 "Enable uuid compression",
 							 &ts_guc_enable_uuid_compression,
-							 false,
+							 true,
 							 PGC_USERSET,
 							 0,
 							 NULL,
@@ -885,7 +971,7 @@ _guc_init(void)
 	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_compressor_batch_limit"),
 							 "Enable compressor batch limit",
 							 "Enable compressor batch limit for compressors which "
-							 "can go over the allocation limit (1 GB). This feature will"
+							 "can go over the allocation limit (1 GB). This feature will "
 							 "limit those compressors by reducing the size of the batch and thus "
 							 "avoid hitting the limit.",
 							 &ts_guc_compression_enable_compressor_batch_limit,
@@ -929,25 +1015,19 @@ _guc_init(void)
 							 NULL,
 							 NULL,
 							 NULL);
-	/*
-	 * Define the limit on number of invalidation-based refreshes we allow per
-	 * refresh call. If this limit is exceeded, fall back to a single refresh that
-	 * covers the range decided by the min and max invalidated time.
-	 */
-	DefineCustomIntVariable(MAKE_EXTOPTION("materializations_per_refresh_window"),
-							"Max number of materializations per cagg refresh window",
-							"The maximal number of individual refreshes per cagg refresh. If more "
-							"refreshes need to be performed, they are merged into a larger "
-							"single refresh.",
-							&ts_guc_cagg_max_individual_materializations,
-							10,
-							0,
-							INT_MAX,
-							PGC_USERSET,
-							0,
-							NULL,
-							NULL,
-							NULL);
+
+	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_cagg_wal_based_invalidation"),
+							 "Enable experimental invalidations for continuous aggregates using "
+							 "WAL",
+							 "Use WAL to track changes to hypertables for continuous aggregates. "
+							 "This is not meant for production use.",
+							 &ts_guc_enable_cagg_wal_based_invalidation,
+							 false,
+							 PGC_USERSET,
+							 0,
+							 NULL,
+							 NULL,
+							 NULL);
 
 	DefineCustomIntVariable(MAKE_EXTOPTION("cagg_processing_wal_batch_size"),
 							"Batch size when processing WAL entries.",
@@ -1086,6 +1166,20 @@ _guc_init(void)
 							 NULL,
 							 NULL,
 							 NULL);
+
+	DefineCustomBoolVariable(MAKE_EXTOPTION("read_legacy_bloom1_v1"),
+							 "Enable reading the legacy bloom1 version 1 sparse indexes for SELECT "
+							 "queries",
+							 "These legacy indexes might give false negatives if they were built "
+							 "by the TimescaleDB extension compiled with different build options.",
+							 &ts_guc_read_legacy_bloom1_v1,
+							 false,
+							 PGC_USERSET,
+							 0,
+							 NULL,
+							 NULL,
+							 NULL);
+
 	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_columnarscan"),
 							 "Enable columnar-optimized scans for supported access methods",
 							 "A columnar scan replaces sequence scans for columnar-oriented "
@@ -1175,6 +1269,20 @@ _guc_init(void)
 							 NULL,
 							 NULL);
 
+#ifdef TS_DEBUG
+	DefineCustomBoolVariable(MAKE_EXTOPTION("enable_partitioned_hypertables"),
+							 "Enable hypertables using declarative partitioning",
+							 "Enable experimental support for creating hypertables using "
+							 "PostgreSQL's native declarative partitioning",
+							 &ts_guc_enable_partitioned_hypertables,
+							 false,
+							 PGC_USERSET,
+							 0,
+							 NULL,
+							 NULL,
+							 NULL);
+#endif
+
 #ifdef USE_TELEMETRY
 	DefineCustomEnumVariable(MAKE_EXTOPTION("telemetry_level"),
 							 "Telemetry settings level",
@@ -1311,7 +1419,7 @@ _guc_init(void)
 							/* show_hook= */ NULL);
 
 	DefineCustomEnumVariable(/* name= */ MAKE_EXTOPTION("debug_require_batch_sorted_merge"),
-							 /* short_desc= */ "require batch sorted merge in DecompressChunk node",
+							 /* short_desc= */ "require batch sorted merge in ColumnarScan node",
 							 /* long_desc= */ "this is for debugging purposes",
 							 /* valueAddr= */ (int *) &ts_guc_debug_require_batch_sorted_merge,
 							 /* bootValue= */ DRO_Allow,
@@ -1389,7 +1497,7 @@ _guc_init(void)
 	DefineCustomEnumVariable(/* name= */ MAKE_EXTOPTION("debug_require_vector_qual"),
 							 /* short_desc= */
 							 "ensure that non-vectorized or vectorized filters are used in "
-							 "DecompressChunk node",
+							 "ColumnarScan node",
 							 /* long_desc= */
 							 "this is for debugging purposes, to let us check if the vectorized "
 							 "quals are used or not. EXPLAIN differs after PG15 for custom nodes, "
