@@ -24,6 +24,7 @@
 #include <commands/trigger.h>
 #include <executor/spi.h>
 #include <funcapi.h>
+#include <lib/stringinfo.h>
 #include <miscadmin.h>
 #include <nodes/makefuncs.h>
 #include <nodes/memnodes.h>
@@ -53,12 +54,14 @@
 #include "dimension_vector.h"
 #include "error_utils.h"
 #include "errors.h"
+#include "extension.h"
 #include "guc.h"
 #include "hypercube.h"
 #include "hypertable_cache.h"
 #include "indexing.h"
 #include "license_guc.h"
 #include "osm_callbacks.h"
+#include "partition_chunk.h"
 #include "scan_iterator.h"
 #include "scanner.h"
 #include "subspace_store.h"
@@ -1317,7 +1320,8 @@ hypertable_validate_constraints(Oid relid)
 
 		if (form->contype == CONSTRAINT_FOREIGN)
 		{
-			if (ts_hypertable_relid_to_id(form->confrelid) != INVALID_HYPERTABLE_ID)
+			if (ts_hypertable_relid_to_id(form->confrelid) != INVALID_HYPERTABLE_ID &&
+				!is_partitioning_allowed(form->confrelid))
 				ereport(ERROR,
 						(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 						 errmsg("hypertables cannot be used as foreign key references of "
@@ -1767,10 +1771,13 @@ ts_hypertable_create_from_info(Oid table_relid, int32 hypertable_id, uint32 flag
 	switch (get_rel_relkind(table_relid))
 	{
 		case RELKIND_PARTITIONED_TABLE:
-			ereport(ERROR,
-					(errcode(ERRCODE_WRONG_OBJECT_TYPE),
-					 errmsg("table \"%s\" is already partitioned", get_rel_name(table_relid)),
-					 errdetail("It is not possible to turn partitioned tables into hypertables.")));
+			if (!ts_guc_enable_partitioned_hypertables)
+				ereport(ERROR,
+						(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+						 errmsg("table \"%s\" is already partitioned", get_rel_name(table_relid)),
+						 errdetail(
+							 "It is not possible to turn partitioned tables into hypertables.")));
+			break;
 		case RELKIND_MATVIEW:
 		case RELKIND_RELATION:
 			break;
@@ -1794,7 +1801,10 @@ ts_hypertable_create_from_info(Oid table_relid, int32 hypertable_id, uint32 flag
 	/* Check that the table doesn't have any unsupported constraints */
 	hypertable_validate_constraints(table_relid);
 
-	table_has_data = ts_relation_has_tuples(rel);
+	/* No need to check for data in partitioned tables */
+	table_has_data = get_rel_relkind(table_relid) == RELKIND_PARTITIONED_TABLE ?
+						 false :
+						 ts_relation_has_tuples(rel);
 
 	if ((flags & HYPERTABLE_CREATE_MIGRATE_DATA) == 0 && table_has_data)
 		ereport(ERROR,
@@ -2258,13 +2268,36 @@ ts_hypertable_create_compressed(Oid table_relid, int32 hypertable_id)
 	return true;
 }
 
+/*
+ * Construct an expression for a dimensional column which is compatible with the max() function.
+ * Normally, this is just the column name, but in the case of UUIDv7 there is no max() function
+ * defined for the type so in that case the expression extracts the timestamp from the UUID.
+ */
+static const char *
+get_expr_for_dim_max(const char *colname, Oid timetype)
+{
+	if (timetype == UUIDOID)
+	{
+		StringInfoData expr;
+
+		initStringInfo(&expr);
+		appendStringInfo(&expr,
+						 "%s.uuid_timestamp(%s)",
+						 ts_extension_schema_name(),
+						 quote_identifier(colname));
+		return expr.data;
+	}
+
+	return quote_identifier(colname);
+}
+
 /*
  * Get the max value of an open dimension.
  */
 int64
 ts_hypertable_get_open_dim_max_value(const Hypertable *ht, int dimension_index, bool *isnull)
 {
-	StringInfo command;
+	StringInfoData command;
 	const Dimension *dim;
 	int res;
 	bool max_isnull;
@@ -2285,17 +2318,17 @@ ts_hypertable_get_open_dim_max_value(const Hypertable *ht, int dimension_index,
 	 * search_path and instead have to fully schema-qualify
 	 * everything.
 	 */
-	command = makeStringInfo();
-	appendStringInfo(command,
+	initStringInfo(&command);
+	appendStringInfo(&command,
 					 "SELECT pg_catalog.max(%s) FROM %s.%s",
-					 quote_identifier(NameStr(dim->fd.column_name)),
+					 get_expr_for_dim_max(NameStr(dim->fd.column_name), timetype),
 					 quote_identifier(NameStr(ht->fd.schema_name)),
 					 quote_identifier(NameStr(ht->fd.table_name)));
 
 	if (SPI_connect() != SPI_OK_CONNECT)
 		elog(ERROR, "could not connect to SPI");
 
-	res = SPI_execute(command->data, true /* read_only */, 0 /*count*/);
+	res = SPI_execute(command.data, true /* read_only */, 0 /*count*/);
 
 	if (res < 0)
 		ereport(ERROR,
@@ -2303,7 +2336,11 @@ ts_hypertable_get_open_dim_max_value(const Hypertable *ht, int dimension_index,
 				 (errmsg("could not find the maximum time value for hypertable \"%s\"",
 						 get_rel_name(ht->main_table_relid)))));
 
-	Ensure(SPI_gettypeid(SPI_tuptable->tupdesc, 1) == timetype,
+	/* In most cases the result type is the same as the time type. However, with UUIDs we first
+	 * extract the timestamptz so the result type is timestamptz instead. */
+	Oid result_type = timetype == UUIDOID ? TIMESTAMPTZOID : timetype;
+
+	Ensure(SPI_gettypeid(SPI_tuptable->tupdesc, 1) == result_type,
 		   "partition types for result (%d) and dimension (%d) do not match",
 		   SPI_gettypeid(SPI_tuptable->tupdesc, 1),
 		   ts_dimension_get_partition_type(dim));
@@ -2313,7 +2350,7 @@ ts_hypertable_get_open_dim_max_value(const Hypertable *ht, int dimension_index,
 		*isnull = max_isnull;
 
 	int64 max_value =
-		max_isnull ? ts_time_get_min(timetype) : ts_time_value_to_internal(maxdat, timetype);
+		max_isnull ? ts_time_get_min(result_type) : ts_time_value_to_internal(maxdat, result_type);
 
 	res = SPI_finish();
 	if (res != SPI_OK_FINISH)
