@@ -33,3 +33,9 @@ extern TSDLLEXPORT int32 ts_jsonb_get_int32_field(const Jsonb *json, const char
 												  bool *field_found);
 extern TSDLLEXPORT int64 ts_jsonb_get_int64_field(const Jsonb *json, const char *key,
 												  bool *field_found);
+extern TSDLLEXPORT bool ts_jsonb_equal(Jsonb *left, Jsonb *right);
+extern TSDLLEXPORT Jsonb *ts_jsonb_replace_key_value_str_field(Jsonb *jb, const char *key,
+															   const char *old, const char *new,
+															   bool *replaced);
+extern TSDLLEXPORT bool ts_jsonb_has_key_value_str_field(Jsonb *jb, const char *key,
+														 const char *value);
