@@ -26,12 +26,17 @@
 
 #include <math.h>
 
+#include "compat/compat.h"
 #include "loader/lwlocks.h"
 #include "nodes/chunk_append/chunk_append.h"
 #include "planner/planner.h"
 #include "transform.h"
 #include "ts_catalog/chunk_column_stats.h"
 
+#if PG18_GE
+#include <commands/explain_format.h>
+#endif
+
 #define INVALID_SUBPLAN_INDEX (-1)
 #define NO_MATCHING_SUBPLANS (-2)
 
@@ -389,7 +394,7 @@ perform_plan_init(ChunkAppendState *state, EState *estate, int eflags)
 		return;
 	}
 
-	state->subplanstates = palloc0(state->num_subplans * sizeof(PlanState *));
+	state->subplanstates = (PlanState **) palloc0(state->num_subplans * sizeof(PlanState *));
 
 	i = 0;
 	foreach (lc, state->filtered_subplans)
