@@ -25,9 +25,7 @@ SELECT * FROM _timescaledb_catalog.hypertable;
 DROP TABLE "newschema"."newname";
 
 SELECT * FROM _timescaledb_catalog.hypertable;
-\dt  "public".*
-\dt  "_timescaledb_catalog".*
-\dt "_timescaledb_internal".*
+SELECT schema, name FROM test.relation WHERE schema IN ('public', '_timescaledb_catalog', '_timescaledb_internal');
 
 -- Test that renaming ordinary table works
 
