@@ -22,7 +22,7 @@ BEGIN
     FOR constraint_row IN
     SELECT c.conname, h.id AS hypertable_id FROM _timescaledb_catalog.hypertable h INNER JOIN
            pg_constraint c ON (c.conrelid = format('%I.%I', h.schema_name, h.table_name)::regclass)
-        WHERE c.contype != 'c'
+        WHERE c.contype NOT IN ('c','n')
     LOOP
         SELECT count(*) FROM _timescaledb_catalog.chunk c
         WHERE c.hypertable_id = constraint_row.hypertable_id
